@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -213,7 +213,7 @@ static int dun_create_tty(int sk, char *tty, int size)
 		if (stat(tty, &st) < 0) {
 			snprintf(tty, size, "/dev/rfcomm%d", id);
 			if (try--) {
-				usleep(100);
+				usleep(100 * 1000);
 				continue;
 			}
 
