@@ -31,16 +31,18 @@
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/l2cap.h>
+#include <bluetooth/sdp.h>
 
 #include <glib.h>
 
 #include <dbus/dbus.h>
 
-#include "hcid.h"
 #include "dbus.h"
+#include "dbus-helper.h"
+#include "hcid.h"
 #include "dbus-common.h"
+#include "adapter.h"
 #include "dbus-hci.h"
-#include "dbus-adapter.h"
 #include "dbus-error.h"
 #include "dbus-test.h"
 
@@ -93,16 +95,9 @@ static struct audit *audit_new(DBusConnection *conn, DBusMessage *msg,
 	path = dbus_message_get_path(msg);
 	requestor = dbus_message_get_sender(msg);
 
-	audit = malloc(sizeof(struct audit));
-	if (!audit)
-		return NULL;
-	memset(audit, 0, sizeof(struct audit));
+	audit = g_new0(struct audit, 1);
 
-	audit->requestor = strdup(requestor);
-	if (!audit->requestor) {
-		free(audit);
-		return NULL;
-	}
+	audit->requestor = g_strdup(requestor);
 
 	bacpy(&audit->peer, peer);
 	bacpy(&audit->local, local);
@@ -114,28 +109,21 @@ static struct audit *audit_new(DBusConnection *conn, DBusMessage *msg,
 
 static void audit_free(struct audit *audit)
 {
-	free(audit->requestor);
+	g_free(audit->requestor);
 	dbus_connection_unref(audit->conn);
-	free(audit);
+	g_free(audit);
 }
 
 static void send_audit_status(struct audit *audit, const char *name)
 {
-	DBusMessage *signal;
 	char addr[18], *addr_ptr = addr;
 
-	signal = dbus_message_new_signal(audit->adapter_path, TEST_INTERFACE, name);
-	if (!signal) {
-		error("Failed to allocate new D-Bus message");
-		return;
-	}
-
 	ba2str(&audit->peer, addr);
 
-	dbus_message_append_args(signal, DBUS_TYPE_STRING, &addr_ptr,
+	dbus_connection_emit_signal(audit->conn, audit->adapter_path,
+					TEST_INTERFACE, name,
+					DBUS_TYPE_STRING, &addr_ptr,
 					DBUS_TYPE_INVALID);
-
-	send_message_and_unref(audit->conn, signal);
 }
 
 static void audit_requestor_exited(const char *name, struct audit *audit)
@@ -147,7 +135,7 @@ static void audit_requestor_exited(const char *name, struct audit *audit)
 
 		send_audit_status(audit, "AuditRemoteDeviceComplete");
 
-		dbus_connection_get_object_path_data(audit->conn,
+		dbus_connection_get_object_user_data(audit->conn,
 							audit->adapter_path,
 							(void *) &adapter);
 		if (adapter)
@@ -358,7 +346,7 @@ static gboolean l2raw_connect_complete(GIOChannel *io, GIOCondition cond, struct
 		return FALSE;
 	}
 
-	dbus_connection_get_object_path_data(audit->conn, audit->adapter_path,
+	dbus_connection_get_object_user_data(audit->conn, audit->adapter_path,
 						(void *) &adapter);
 	if (adapter)
 		bacpy(&adapter->agents_disabled, BDADDR_ANY);
@@ -645,27 +633,26 @@ static DBusHandlerResult get_l2cap_mtu_size(DBusConnection *conn,
 	return send_message_and_unref(conn, reply);
 }
 
-static struct service_data methods[] = {
-	{ "AuditRemoteDevice",		audit_remote_device		},
-	{ "CancelAuditRemoteDevice",	cancel_audit_remote_device	},
-	{ "GetL2capFeatureMask",	get_l2cap_feature_mask		},
-	{ "GetL2capMtuSize",		get_l2cap_mtu_size		},
-	{ NULL, NULL }
+static DBusMethodVTable test_methods[] = {
+	{ "AuditRemoteDevice",		audit_remote_device,
+		"s",	""	},
+	{ "CancelAuditRemoteDevice",	cancel_audit_remote_device,
+		"s",	""	},
+	{ "GetL2capFeatureMask",	get_l2cap_feature_mask,
+		"s",	"u"	},
+	{ "GetL2capMtuSize",		get_l2cap_mtu_size,
+		"s",	"q"	},
+	{ NULL, NULL, NULL, NULL }
 };
 
-DBusHandlerResult handle_test_method(DBusConnection *conn, DBusMessage *msg, void *data)
+dbus_bool_t test_init(DBusConnection *conn, const char *path)
 {
-	service_handler_func_t handler;
-
 	if (!hcid_dbus_use_experimental())
-		return error_unknown_method(conn, msg);
+		return TRUE;
 
-	handler = find_service_handler(methods, msg);
-
-	if (handler)
-		return handler(conn, msg, data);
-
-	return error_unknown_method(conn, msg);
+	return dbus_connection_register_interface(conn, path, TEST_INTERFACE,
+							test_methods,
+							NULL, NULL);
 }
 
 void process_audits_list(const char *adapter_path)
@@ -688,7 +675,7 @@ void process_audits_list(const char *adapter_path)
 
 		adapter = NULL;
 
-		dbus_connection_get_object_path_data(audit->conn,
+		dbus_connection_get_object_user_data(audit->conn,
 							audit->adapter_path,
 							(void *) &adapter);
 
