@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -31,13 +31,14 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include <time.h>
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 
+#include <glib.h>
+
 #include <bluetooth/bluetooth.h>
 
 #include "textfile.h"
@@ -121,7 +122,7 @@ int write_local_name(bdaddr_t *bdaddr, char *name)
 
 	memset(str, 0, sizeof(str));
 	for (i = 0; i < 248 && name[i]; i++)
-		if (name[i] < 32 || name[i] == 127)
+		if ((unsigned char) name[i] < 32 || name[i] == 127)
 			str[i] = '.';
 		else
 			str[i] = name[i];
@@ -232,7 +233,7 @@ int write_device_name(bdaddr_t *local, bdaddr_t *peer, char *name)
 
 	memset(str, 0, sizeof(str));
 	for (i = 0; i < 248 && name[i]; i++)
-		if (name[i] < 32 || name[i] == 127)
+		if ((unsigned char) name[i] < 32 || name[i] == 127)
 			str[i] = '.';
 		else
 			str[i] = name[i];
@@ -267,6 +268,70 @@ int read_device_name(bdaddr_t *local, bdaddr_t *peer, char *name)
 	return 0;
 }
 
+int write_l2cap_info(bdaddr_t *local, bdaddr_t *peer,
+			uint16_t mtu_result, uint16_t mtu,
+			uint16_t mask_result, uint32_t mask)
+{
+	char filename[PATH_MAX + 1], addr[18], str[18];
+
+	if (mask_result)
+		snprintf(str, sizeof(str), "%d -1", mtu_result ? -1 : mtu);
+	else
+		snprintf(str, sizeof(str), "%d 0x%08x", mtu_result ? -1 : mtu, mask);
+
+	create_filename(filename, PATH_MAX, local, "l2cap");
+
+	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	ba2str(peer, addr);
+	return textfile_put(filename, addr, str);
+}
+
+int read_l2cap_info(bdaddr_t *local, bdaddr_t *peer,
+			uint16_t *mtu_result, uint16_t *mtu,
+			uint16_t *mask_result, uint32_t *mask)
+{
+	char filename[PATH_MAX + 1], addr[18], *str, *space, *msk;
+
+	create_filename(filename, PATH_MAX, local, "l2cap");
+
+	ba2str(peer, addr);
+	str = textfile_get(filename, addr);
+	if (!str)
+		return -ENOENT;
+
+	space = strchr(str, ' ');
+	if (!space) {
+		free(str);
+		return -ENOENT;
+	}
+
+	msk = space + 1;
+	*space = '\0';
+
+	if (mtu_result && mtu) {
+		if (str[0] == '-')
+			*mtu_result = 0x0001;
+		else {
+			*mtu_result = 0;
+			*mtu = (uint16_t) strtol(str, NULL, 0);
+		}
+	}
+
+	if (mask_result && mask) {
+		if (msk[0] == '-')
+			*mask_result = 0x0001;
+		else {
+			*mask_result = 0;
+			*mask = (uint32_t) strtol(msk, NULL, 16);
+		}
+	}
+
+	free(str);
+
+	return 0;
+}
+
 int write_version_info(bdaddr_t *local, bdaddr_t *peer, uint16_t manufacturer, uint8_t lmp_ver, uint16_t lmp_subver)
 {
 	char filename[PATH_MAX + 1], addr[18], str[16];
@@ -423,3 +488,152 @@ int read_pin_code(bdaddr_t *local, bdaddr_t *peer, char *pin)
 
 	return len;
 }
+
+static GSList *service_string_to_list(char *services)
+{
+	GSList *l = NULL;
+	char *start = services;
+	int i, finished = 0;
+
+	for (i = 0; !finished; i++) {
+		if (services[i] == '\0')
+			finished = 1;
+
+		if (services[i] == ' ' || services[i] == '\0') {
+			services[i] = '\0';
+			l = g_slist_append(l, start);
+			start = services + i + 1;
+		}
+	}
+
+	return l;
+}
+
+static char *service_list_to_string(GSList *services)
+{
+	char str[1024];
+	int len = 0;
+
+	if (!services)
+		return g_strdup("");
+
+	memset(str, 0, sizeof(str));
+
+	while (services) {
+		int ret;
+		char *ident = services->data;
+
+		ret = snprintf(str + len, sizeof(str) - len - 1, "%s%s",
+				ident, services->next ? " " : "");
+
+		if (ret > 0)
+			len += ret;
+
+		services = services->next;
+	}
+
+	return g_strdup(str);
+}
+
+int write_trust(bdaddr_t *local, const char *addr, const char *service,
+		gboolean trust)
+{
+	char filename[PATH_MAX + 1], *str;
+	GSList *services = NULL, *match;
+	gboolean trusted;
+	int ret;
+
+	create_filename(filename, PATH_MAX, local, "trusts");
+
+	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	str = textfile_caseget(filename, addr);
+	if (str)
+		services = service_string_to_list(str);
+
+	match = g_slist_find_custom(services, service, (GCompareFunc) strcmp);
+	trusted = match ? TRUE : FALSE;
+
+	/* If the old setting is the same as the requested one, we're done */
+	if (trusted == trust) {
+		g_slist_free(services);
+		if (str)
+			free(str);
+		return 0;
+	}
+
+	if (trust)
+		services = g_slist_append(services, (void *) service);
+	else
+		services = g_slist_remove(services, match->data);
+
+	/* Remove the entry if the last trusted service was removed */
+	if (!trust && !services)
+		ret = textfile_casedel(filename, addr);
+	else {
+		char *new_str = service_list_to_string(services);
+		ret = textfile_caseput(filename, addr, new_str);
+		free(new_str);
+	}
+
+	g_slist_free(services);
+
+	if (str)
+		free(str);
+
+	return ret;
+}
+
+gboolean read_trust(bdaddr_t *local, const char *addr, const char *service)
+{
+	char filename[PATH_MAX + 1], *str;
+	GSList *services;
+	gboolean ret;
+
+	create_filename(filename, PATH_MAX, local, "trusts");
+
+	str = textfile_caseget(filename, addr);
+	if (!str)
+		return FALSE;
+
+	services = service_string_to_list(str);
+
+	if (g_slist_find_custom(services, service, (GCompareFunc) strcmp))
+		ret = TRUE;
+	else
+		ret = FALSE;
+
+	g_slist_free(services);
+	free(str);
+
+	return ret;
+}
+
+struct trust_list {
+	GSList *trusts;
+	const char *service;
+};
+
+static void append_trust(char *key, char *value, void *data)
+{
+	struct trust_list *list = data;
+
+	if (strstr(value, list->service))
+		list->trusts = g_slist_append(list->trusts, g_strdup(key));
+}
+
+GSList *list_trusts(bdaddr_t *local, const char *service)
+{
+	char filename[PATH_MAX + 1];
+	struct trust_list list;
+
+	create_filename(filename, PATH_MAX, local, "trusts");
+
+	list.trusts = NULL;
+	list.service = service;
+
+	if (textfile_foreach(filename, append_trust, &list) < 0)
+		return NULL;
+
+	return list.trusts;
+}
