@@ -44,8 +44,10 @@ int service_register_req(sdp_req_t *req, sdp_buf_t *rsp);
 int service_update_req(sdp_req_t *req, sdp_buf_t *rsp);
 int service_remove_req(sdp_req_t *req, sdp_buf_t *rsp);
 
-void register_public_browse_group(int public);
-void register_server_service(int public);
+void register_public_browse_group(void);
+void register_server_service(void);
+void register_device_id(const uint16_t vendor, const uint16_t product,
+						const uint16_t version);
 
 typedef struct {
 	uint32_t timestamp;
@@ -78,10 +80,15 @@ uint32_t sdp_get_time();
 
 #define SDP_SERVER_COMPAT (1 << 0)
 #define SDP_SERVER_MASTER (1 << 1)
-#define SDP_SERVER_PUBLIC (1 << 2)
 
-int start_sdp_server(uint16_t mtu, uint32_t flags);
+int start_sdp_server(uint16_t mtu, const char *did, uint32_t flags);
 void stop_sdp_server(void);
 
 int add_record_to_server(sdp_record_t *rec);
-void remove_record_from_server(uint32_t handle);
+int remove_record_from_server(uint32_t handle);
+
+typedef void (*service_classes_callback_t) (const bdaddr_t *bdaddr, uint8_t value);
+
+uint8_t get_service_classes(const bdaddr_t *bdaddr);
+void set_service_classes_callback(service_classes_callback_t callback);
+void create_ext_inquiry_response(const char *name, uint8_t *data);
