@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -253,11 +253,6 @@ static void cmd_up(int ctl, int hdev, char *opt)
 						hdev, strerror(errno), errno);
 		exit(1);
 	}
-	if (ioctl(ctl, HCIGETDEVINFO, (void *) &di) < 0)
-		return;
-	if (hci_test_bit(HCI_RAW, &di.flags))
-		return;
-	cmd_scan(ctl, hdev, "piscan");
 }
 
 static void cmd_down(int ctl, int hdev, char *opt)
@@ -438,36 +433,11 @@ static void cmd_name(int ctl, int hdev, char *opt)
 	}
 
 	if (opt) {
-		uint8_t fec = 0, data[240];
-		int len, eir = 0;
-
-		if (di.features[6] & LMP_EXT_INQ) {
-			if (hci_read_ext_inquiry_response(dd, &fec, data, 1000) == 0)
-				eir = 1;
-		}
-
-		memset(data, 0, sizeof(data));
-		len = strlen(opt);
-		if (len > 48) {
-			len = 48;
-			data[1] = 0x08;
-		} else
-			data[1] = 0x09;
-		data[0] = len + 1;
-		memcpy(data + 2, opt, len);
-
 		if (hci_write_local_name(dd, opt, 2000) < 0) {
 			fprintf(stderr, "Can't change local name on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
-
-		if (eir) {
-			if (hci_write_ext_inquiry_response(dd, fec, data, 2000) < 0) {
-				fprintf(stderr, "Can't set extended inquiry response on hci%d: %s (%d)\n",
-						hdev, strerror(errno), errno);
-			}
-		}
 	} else {
 		char name[249];
 		int i;
@@ -478,9 +448,11 @@ static void cmd_name(int ctl, int hdev, char *opt)
 			exit(1);
 		}
 
-		for (i = 0; i < 248 && name[i]; i++)
-			if (!isprint(name[i]))
+		for (i = 0; i < 248 && name[i]; i++) {
+			if ((unsigned char) name[i] < 32 || name[i] == 127)
 				name[i] = '.';
+		}
+
 		name[248] = '\0';
 
 		print_dev_hdr(&di);
@@ -597,16 +569,53 @@ static char *get_minor_device_name(int major, int minor)
 			return "Gaming/Toy";
 		}
 		break;
-	case 5:	/* peripheral */
-		switch(minor) {
+	case 5:	/* peripheral */ {
+		static char cls_str[48];
+		
+		cls_str[0] = '\0';
+
+		switch(minor & 48) {
 		case 16:
-			return "Keyboard";
+			strncpy(cls_str, "Keyboard", sizeof(cls_str));
+			break;
 		case 32:
-			return "Pointing device";
+			strncpy(cls_str, "Pointing device", sizeof(cls_str));
+			break;
 		case 48:
-			return "Combo keyboard/pointing device";
+			strncpy(cls_str, "Combo keyboard/pointing device", sizeof(cls_str));
+			break;
 		}
-		break;
+		if((minor & 15) && (strlen(cls_str) > 0))
+			strcat(cls_str, "/");
+
+		switch(minor & 15) {
+		case 0:
+			break;
+		case 1:
+			strncat(cls_str, "Joystick", sizeof(cls_str) - strlen(cls_str));
+			break;
+		case 2:
+			strncat(cls_str, "Gamepad", sizeof(cls_str) - strlen(cls_str));
+			break;
+		case 3:
+			strncat(cls_str, "Remote control", sizeof(cls_str) - strlen(cls_str));
+			break;
+		case 4:
+			strncat(cls_str, "Sensing device", sizeof(cls_str) - strlen(cls_str));
+			break;
+		case 5:
+			strncat(cls_str, "Digitizer tablet", sizeof(cls_str) - strlen(cls_str));
+			break;
+		case 6:
+			strncat(cls_str, "Card reader", sizeof(cls_str) - strlen(cls_str));
+			break;
+		default:
+			strncat(cls_str, "(reserved)", sizeof(cls_str) - strlen(cls_str));
+			break;
+		}
+		if(strlen(cls_str) > 0)
+			return cls_str;
+	}
 	case 6:	/* imaging */
 		if (minor & 4)
 			return "Display";
@@ -617,6 +626,34 @@ static char *get_minor_device_name(int major, int minor)
 		if (minor & 32)
 			return "Printer";
 		break;
+	case 7: /* wearable */
+		switch(minor) {
+		case 1:
+			return "Wrist Watch";
+		case 2:
+			return "Pager";
+		case 3:
+			return "Jacket";
+		case 4:
+			return "Helmet";
+		case 5:
+			return "Glasses";
+		}
+		break;
+	case 8: /* toy */
+		switch(minor) {
+		case 1:
+			return "Robot";
+		case 2:
+			return "Vehicle";
+		case 3:
+			return "Doll / Action Figure";
+		case 4:
+			return "Controller";
+		case 5:
+			return "Game";
+		}
+		break;
 	case 63:	/* uncategorised */
 		return "";
 	}
@@ -823,6 +860,36 @@ static void cmd_delkey(int ctl, int hdev, char *opt)
 	hci_close_dev(dd);
 }
 
+static void cmd_oob_data(int ctl, int hdev, char *opt)
+{
+	uint8_t hash[16], randomizer[16];
+	int i, dd;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (hci_read_local_oob_data(dd, hash, randomizer, 1000) < 0) {
+		fprintf(stderr, "Can't read local OOB data on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	print_dev_hdr(&di);
+	printf("\tOOB Hash:  ");
+	for (i = 0; i < 16; i++)
+		printf(" %02x", hash[i]);
+	printf("\n\tRandomizer:");
+	for (i = 0; i < 16; i++)
+		printf(" %02x", randomizer[i]);
+	printf("\n");
+
+	hci_close_dev(dd);
+}
+
 static void cmd_commands(int ctl, int hdev, char *opt)
 {
 	uint8_t cmds[64];
@@ -850,7 +917,7 @@ static void cmd_commands(int ctl, int hdev, char *opt)
 		printf("%s Octet %-2d = 0x%02x (Bit",
 			i ? "\t\t ": "\tCommands:", i, cmds[i]);
 		for (n = 0; n < 8; n++)
-			if (hci_test_bit(n, &cmds[i]))
+			if (cmds[i] & (1 << n))
 				printf(" %d", n);
 		printf(")\n");
 	}
@@ -1002,19 +1069,39 @@ static void cmd_inq_data(int ctl, int hdev, char *opt)
 			len = *ptr++;
 			type = *ptr++;
 			switch (type) {
+			case 0x01:
+				printf("\tFlags:");
+				for (i = 0; i < len - 1; i++)
+					printf(" 0x%2.2x", *((uint8_t *) (ptr + i)));
+				printf("\n");
+				break;
+			case 0x02:
+			case 0x03:
+				printf("\t%s service classes:",
+					type == 0x02 ? "Shortened" : "Complete");
+				for (i = 0; i < (len - 1) / 2; i++) {
+					uint16_t val = btohs(bt_get_unaligned((uint16_t *) (ptr + (i * 2))));
+					printf(" 0x%4.4x", val);
+				}
+				printf("\n");
+				break;
 			case 0x08:
 			case 0x09:
 				str = malloc(len);
 				if (str) {
 					snprintf(str, len, "%s", ptr);
-					for (i = 0; i < len - 1; i++)
-						if (!isprint(str[i]))
+					for (i = 0; i < len - 1; i++) {
+						if ((unsigned char) str[i] < 32 || str[i] == 127)
 							str[i] = '.';
+					}
 					printf("\t%s local name: \'%s\'\n",
 						type == 0x08 ? "Shortened" : "Complete", str);
 					free(str);
 				}
 				break;
+			case 0x0a:
+				printf("\tTX power level: %d\n", *((uint8_t *) ptr));
+				break;
 			default:
 				printf("\tUnknown type 0x%02x with %d bytes data\n",
 								type, len - 1);
@@ -1301,6 +1388,39 @@ static void cmd_afh_mode(int ctl, int hdev, char *opt)
 	}
 }
 
+static void cmd_ssp_mode(int ctl, int hdev, char *opt)
+{
+	int dd;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (opt) {
+		uint8_t mode = atoi(opt);
+
+		if (hci_write_simple_pairing_mode(dd, mode, 2000) < 0) {
+			fprintf(stderr, "Can't set Simple Pairing mode on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+	} else {
+		uint8_t mode;
+
+		if (hci_read_simple_pairing_mode(dd, &mode, 1000) < 0) {
+			fprintf(stderr, "Can't read Simple Pairing mode on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+
+		print_dev_hdr(&di);
+		printf("\tSimple Pairing mode: %s\n", mode == 1 ? "Enabled" : "Disabled");
+	}
+}
+
 static void print_rev_ericsson(int dd)
 {
 	struct hci_request rq;
@@ -1402,6 +1522,7 @@ static void cmd_revision(int ctl, int hdev, char *opt)
 	print_dev_hdr(&di);
 	switch (ver.manufacturer) {
 	case 0:
+	case 37:
 	case 48:
 		print_rev_ericsson(dd);
 		break;
@@ -1506,10 +1627,12 @@ static struct {
 	{ "pageparms",	cmd_page_parms,	"[win:int]",	"Get/Set page scan window and interval" },
 	{ "pageto",	cmd_page_to,	"[to]",		"Get/Set page timeout" },
 	{ "afhmode",	cmd_afh_mode,	"[mode]",	"Get/Set AFH mode" },
+	{ "sspmode",	cmd_ssp_mode,	"[mode]",	"Get/Set Simple Pairing Mode" },
 	{ "aclmtu",	cmd_aclmtu,	"<mtu:pkt>",	"Set ACL MTU and number of packets" },
 	{ "scomtu",	cmd_scomtu,	"<mtu:pkt>",	"Set SCO MTU and number of packets" },
 	{ "putkey",	cmd_putkey,	"<bdaddr>",	"Store link key on the device" },
 	{ "delkey",	cmd_delkey,	"<bdaddr>",	"Delete link key from the device" },
+	{ "oobdata",	cmd_oob_data,	0,		"Display local OOB data" },
 	{ "commands",	cmd_commands,	0,		"Display supported commands" },
 	{ "features",	cmd_features,	0,		"Display device features" },
 	{ "version",	cmd_version,	0,		"Display version information" },
