@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -224,7 +224,11 @@ static struct device_id device_list[] = {
 	{ HCI, 0x046d, 0xc704, switch_logitech },
 	{ HCI, 0x046d, 0xc705, switch_logitech },
 	{ HCI, 0x046d, 0xc70a, switch_logitech },	/* Logitech diNovo mouse */
-	{ HCI, 0x046d, 0xc70e, switch_logitech },	/* logitech diNovo keyboard */
+	{ HCI, 0x046d, 0xc70b, switch_logitech },	/* Logitech diNovo Laser keyboard */
+	{ HCI, 0x046d, 0xc70c, switch_logitech },	/* Logitech diNovo Laser mouse */
+	{ HCI, 0x046d, 0xc70e, switch_logitech },	/* Logitech diNovo keyboard */
+	{ HCI, 0x046d, 0xc713, switch_logitech },	/* Logitech diNovo Edge */
+	{ HCI, 0x046d, 0xc714, switch_logitech },	/* Logitech diNovo Edge */
 	{ -1 }
 };
 
